﻿using Elight.Entity;
using Elight.Logic;
using Elight.Utility;
using Elight.Utility.Code;
using Elight.Utility.Extensions;
using Elight.Utility.logs;
using java.util;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;

namespace _24Hour.Controllers.Common
{
    /// <summary>
    /// 云柜详情服务
    /// </summary>
    [Authorize]
    [ApiController]
    [Route("api/Hearing")]
    public class CloudCabinetController : Controller
    {
        #region Identity
        private readonly SqlSugarClient _db;//数据库
        private readonly WriteSysLog _logs;//操作日志
        App_Sys_UserModel _userdata = new App_Sys_UserModel();//当前用户
        private readonly ILogger<LoginController> _logger;//日志
        DateTime unixStartTime = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc);
        Result result = new Result();
        public CloudCabinetController(ILogger<LoginController> logger, SqlSugarClient db, WriteSysLog logs, User user)
        {
            _logger = logger;
            _db = db;
            _logs = logs;
            _userdata = user.Userdata();
        }

        #endregion

        #region 云柜增删改查

        /// <summary>
        /// app--云柜分页查询
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("App_Querycabinet")]
        public async Task<Result> App_Querycabinet(App_CloudCabinetInput cabinetdata)
        {
            RefAsync<int> totalNumber = 0;//总数据
            //单位
            var _Unitdata = _db.Queryable<App_Sys_UnitModel>().Where(q => q.IsDelete == 0).ToList();
            //查询云柜详情
            var list = await _db.Queryable<App_CloudCabinetModel>()
                .WhereIF(cabinetdata.sfzh.NotNull(), q => q.sfzh.Contains(cabinetdata.sfzh))
                .WhereIF(cabinetdata.xm.NotNull(), q => q.xm.Contains(cabinetdata.xm))
                .WhereIF(cabinetdata.lxhm.NotNull(), q => q.lxhm.Contains(cabinetdata.lxhm))
                .WhereIF(cabinetdata.zt.NotNull(), q => q.zt.Contains(cabinetdata.zt))
                .WhereIF(cabinetdata.dwbm.NotNull(), q => q.dwbm.Contains(cabinetdata.dwbm))
                .Where(q => q.sfzh.Contains(_userdata.cardId)|| q.sfzh.Contains(_userdata.phone))
                .ToPageListAsync(cabinetdata.PageIndex, cabinetdata.PageSize, totalNumber);
            cabinetdata.RowsCount = totalNumber;

            list.ForEach(q =>
            {
                var da = _Unitdata.Where(x => x.unitCode == q.dwbm).FirstOrDefault();
                q.dwbm = da?.unitname;
            });
            var data = new QueryResult<App_CloudCabinetModel>(cabinetdata, list.OrderByDescending(q => long.Parse(q.cjsj)).ToList());
            result.IsSucceed = true;
            result.result = data;
            return result;
        }

        /// <summary>
        /// 云柜分页查询
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Querycabinet")]
        public async Task<Result> Querycabinet(App_CloudCabinetInput cabinetdata)
        {
            //查询云柜详情
            var list = await _db.Queryable<App_CloudCabinetModel>()
                .WhereIF(cabinetdata.sfzh.NotNull(), q => q.sfzh.Contains(cabinetdata.sfzh))
                .WhereIF(cabinetdata.xm.NotNull(), q => q.xm.Contains(cabinetdata.xm))
                .WhereIF(cabinetdata.lxhm.NotNull(), q => q.lxhm.Contains(cabinetdata.lxhm))
                .WhereIF(cabinetdata.zt.NotNull(), q => q.zt.Contains(cabinetdata.zt))
                .WhereIF(cabinetdata.StartTime != null && cabinetdata.EndTime != null, q => unixStartTime.AddMilliseconds(long.Parse(q.cjsj)) >= cabinetdata.StartTime && unixStartTime.AddMilliseconds(long.Parse(q.cjsj)) < cabinetdata.EndTime.Value.AddDays(1))
                .ToPageListAsync(cabinetdata.PageIndex, cabinetdata.PageSize);
            var data = new QueryResult<App_CloudCabinetModel>(cabinetdata, list.OrderByDescending(q => long.Parse(q.cjsj)).ToList());
            result.IsSucceed = true;
            result.result = data;
            return result;
        }
        /// <summary>
        /// 添加云柜
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Addcabinet")]
        public async Task<Result> Addcabinet(App_CloudCabinetModel cabinetdata)
        {
            try
            {
                _db.BeginTran();
                cabinetdata.Id = Guid.NewGuid().ToString();
                var num = await _db.Insertable(cabinetdata).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "添加成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "添加云柜");
            }
            _logs.WriteSysLogadd("云柜管理", "添加云柜", result, _db);
            return result;
        }

        /// <summary>
        /// 修改云柜
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Updatecabinet")]
        public async Task<Result> Updatecabinet(App_CloudCabinetModel cabinetdata)
        {
            try
            {
                _db.BeginTran();
                var num = await _db.Updateable(cabinetdata).IgnoreColumns(ignoreAllNullColumns: true).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "修改成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "修改云柜");
            }
            _logs.WriteSysLogadd("云柜管理", "修改云柜", result, _db);
            return result;
        }

        /// <summary>
        /// 删除云柜
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Deletecabinet")]
        public async Task<Result> Deletecabinet(CurrencyDelete Currency)
        {
            try
            {
                var Deletelist = await _db.Queryable<App_CloudCabinetModel>().In(q => q.Id, Currency.id).ToListAsync();
                _db.BeginTran();
                var num = await _db.Deleteable(Deletelist).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "删除成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "删除云柜");
            }
            _logs.WriteSysLogadd("云柜管理", "删除云柜", result, _db);
            return result;
        }
        #endregion

    }
}
