﻿using Elight.Entity;
using Elight.Logic;
using Elight.Utility;
using Elight.Utility.Code;
using Elight.Utility.Extensions;
using Elight.Utility.logs;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;

namespace _24Hour.Controllers.Common
{
    /// <summary>
    /// 远程会见
    /// </summary>
    [Authorize]
    [ApiController]
    [Route("api/Remote")]
    public class RemoteController : Controller
    {
        #region Identity
        private readonly IConfiguration _configuration;
        private readonly SqlSugarClient _db;//数据库
        private readonly WriteSysLog _logs;//操作日志
        App_Sys_UserModel _userdata = new App_Sys_UserModel();//当前用户
        private readonly ILogger<LoginController> _logger;//日志
        Result result = new Result();
        public RemoteController(ILogger<LoginController> logger, SqlSugarClient db, WriteSysLog logs, User user, IConfiguration configuration)
        {
            _logger = logger;
            _db = db;
            _logs = logs;
            _userdata = user.Userdata();
            _configuration = configuration;
        }

        #endregion


        #region 远程会见管理

        /// <summary>
        /// app----根据当前登录人查询远程会见信息分页查询
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("QueryRemoteuser")]
        public async Task<Result> QueryRemoteuser(App_RemoteInput Remotedata)
        {
            RefAsync<int> totalNumber = 0;//总数据
            var Receptionlist = new List<App_RemoteModel>();
            //查询远程会见记录
            var list = await _db.Queryable<App_RemoteModel>()
                .WhereIF(Remotedata.Code != null, q => q.Code.Contains(Remotedata.Code))
                .WhereIF(Remotedata.name != null, q => q.name.Contains(Remotedata.name))
                .WhereIF(Remotedata.unitId.NotNull(), q => q.unitCode.Contains(Remotedata.unitId))
                .WhereIF(Remotedata.meetwitname != null, q => q.meetwitname.Contains(Remotedata.meetwitname))
                .WhereIF(Remotedata.state != null, q => q.state == Remotedata.state)
                .Where(q => q.IsDeleted == 0 && q.createuserId == _userdata.Id).ToPageListAsync(Remotedata.PageIndex, Remotedata.PageSize, totalNumber);
            Remotedata.RowsCount = totalNumber;
            var data = new QueryResult<App_RemoteModel>(Remotedata, list.OrderByDescending(q => q.creationtime).ToList());
            result.IsSucceed = true;
            result.result = data;
            return result;
        }

        /// <summary>
        /// 远程会见分页查询
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("QueryRemote")]
        public async Task<Result> QueryRemote(App_RemoteInput Remotedata)
        {
            RefAsync<int> totalNumber = 0;//总数据
            var Receptionlist = new List<App_RemoteModel>();
            //查询远程会见记录
            var list = await _db.Queryable<App_RemoteModel>()
                .WhereIF(Remotedata.Code != null, q => q.Code.Contains(Remotedata.Code))
                .WhereIF(Remotedata.name != null, q => q.name.Contains(Remotedata.name))
                .WhereIF(Remotedata.meetwitname != null, q => q.meetwitname.Contains(Remotedata.meetwitname))
                .WhereIF(Remotedata.state != null, q => q.state == Remotedata.state)
                .WhereIF(Remotedata.StartTime != null && Remotedata.EndTime != null, q => q.sttime >= Remotedata.StartTime && q.sttime < Remotedata.EndTime.Value.AddDays(1))
                .Where(q => q.IsDeleted == 0&&q.unitCode == _userdata.unitCode).ToPageListAsync(Remotedata.PageIndex, Remotedata.PageSize, totalNumber);
            Remotedata.RowsCount = totalNumber;
            var data = new QueryResult<App_RemoteModel>(Remotedata, list.OrderByDescending(q => q.creationtime).ToList());
            result.IsSucceed = true;
            result.result = data;
            return result;
        }
        /// <summary>
        /// 添加远程会见
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("AddRemote")]
        public async Task<Result> AddRemote(App_RemoteModel Remotedata)
        {
            try
            {
                _db.BeginTran();
                Remotedata.Id = Guid.NewGuid().ToString();
                //Remotedata.unitId = _userdata.unitCode.ToString();
                Remotedata.Code=_logs.GenerateRandomCode(9);
                Remotedata.createuserId = _userdata.Id.ToString();
                Remotedata.appurl = $"{_configuration.GetSection("Videoaddress:rtmp").Value}{Remotedata.Code}";
                Remotedata.createusername = _userdata.name;
                var num = await _db.Insertable(Remotedata).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "添加成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "远程会见添加");
            }
            _logs.WriteSysLogadd("远程会见管理", "添加远程会见", result, _db);
            return result;
        }

        /// <summary>
        /// 修改远程会见
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("UpdateRemote")]
        public async Task<Result> UpdateRemote(App_RemoteModel Remotedata)
        {
            try
            {
                _db.BeginTran();
                var num = await _db.Updateable(Remotedata).IgnoreColumns(ignoreAllNullColumns: true).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "修改成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "远程会见修改");
            }
            _logs.WriteSysLogadd("远程会见管理", "修改远程会见", result, _db);
            return result;
        }

        /// <summary>
        /// 删除远程会见
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("DeleteRemote")]
        public async Task<Result> DeleteRemote(CurrencyDelete Currency)
        {
            try
            {
                _db.BeginTran();
                var Receptionlist = await _db.Queryable<App_RemoteModel>().In(q => q.Id, Currency.id).ToListAsync();
                Receptionlist.ForEach(q =>
                {
                    q.IsDeleted = 1;
                });
                var num = await _db.Updateable(Receptionlist).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "删除成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "远程会见删除");
            }
            _logs.WriteSysLogadd("远程会见管理", "删除远程会见", result, _db);
            return result;
        }
        /// <summary>
        /// 远程会见状态更改
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("Updateagree_state")]
        public async Task<Result> Updateagree_state(App_RemoteState data)
        {
            try
            {
                var Receptionlist = await _db.Queryable<App_RemoteModel>().Where(q => q.Id==data.Id).ToListAsync();
                Receptionlist.ForEach(q => {
                    q.state = (int)data.state;
                    if (data.sttime != null) q.sttime = data.sttime;
                    if (data.ettime != null) q.ettime = data.ettime;
                    q.reason = data.reason;
                });
                _db.BeginTran();
                var num = await _db.Updateable(Receptionlist).IgnoreColumns(ignoreAllNullColumns: true).UpdateColumns(it => new { it.state, it.sttime,it.ettime,it.reason }).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "更改成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "远程会见更改状态");
            }
            _logs.WriteSysLogadd("远程会见管理", "取消远程会见", result, _db);
            return result;
        }
        #endregion

        #region 远程会见分配会见人接口

        /// <summary>
        /// 修改远程会见预约办理人
        /// </summary>
        /// <param name="Id">远程会见Id</param>
        /// <param name="meetwitId">办理人Id</param>
        /// <param name="meetwitname">办理人名称</param>
        /// <returns></returns>
        [HttpGet]
        [Route("UpdateRemoteuser")]
        public async Task<Result> UpdateRemoteuser(string? Id, string? meetwitId,string? meetwitname)
        {
            try
            {
                var Lawyerbol = await _db.Queryable<App_RemoteModel>().Where(q => q.Id == Id).ToListAsync();
                if (Lawyerbol.Count() > 0)
                {
                    Lawyerbol.FirstOrDefault().meetwitId = meetwitId;
                    Lawyerbol.FirstOrDefault().meetwitId = meetwitname;
                    _db.BeginTran();
                    var num = await _db.Updateable(Lawyerbol.FirstOrDefault()).UpdateColumns(it => new { it.meetwitId,it.meetwitname }).ExecuteCommandAsync();
                    _db.CommitTran();
                    if (num > 0)
                    {
                        result.IsSucceed = true;
                        result.result = "修改成功";
                    }
                }
                else
                {
                    result.IsSucceed = false;
                    result.result = "未找到预约信息";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "修改律师服务预约办理人");
            }
            _logs.WriteSysLogadd("律师服务", "修改律师服务预约办理人", result, _db);
            return result;
        }

        /// <summary>
        /// 修改远程会见状态更改
        /// </summary>
        /// <param name="Id">远程会见Id</param>
        /// <param name="meetwitId">办理人Id</param>
        /// <param name="meetwitname">办理人名称</param>
        /// <returns></returns>
        [HttpPost]
        [Route("UpdateRemotestate")]
        public async Task<Result> UpdateRemotestate(App_Remoteput Remoteput)
        {
            try
            {
                var Lawyerbol = await _db.Queryable<App_RemoteModel>().Where(q => q.Id == Remoteput.Id).ToListAsync();
                if (Lawyerbol.Count() > 0)
                {
                    Lawyerbol.FirstOrDefault().state = Remoteput.state;
                    if(Remoteput.sttime!=null)
                    Lawyerbol.FirstOrDefault().sttime = Remoteput.sttime;
                    if (Remoteput.ettime != null)
                        Lawyerbol.FirstOrDefault().ettime = Remoteput.ettime;
                    if (Remoteput.reason.NotNull())
                    Lawyerbol.FirstOrDefault().reason = Remoteput.reason;
                    Lawyerbol.FirstOrDefault().acceptancetime = DateTime.Now;
                    _db.BeginTran();
                    var num = await _db.Updateable(Lawyerbol.FirstOrDefault()).ExecuteCommandAsync();
                    _db.CommitTran();
                    if (num > 0)
                    {
                        result.IsSucceed = true;
                        result.result = "修改成功";
                    }
                }
                else
                {
                    result.IsSucceed = false;
                    result.result = "未找到预约信息";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
                LogService.WriteLog(ex, "修改律师服务预约办理人");
            }
            _logs.WriteSysLogadd("律师服务", "修改律师服务预约办理人", result, _db);
            return result;
        }
        #endregion
    }
}
