﻿using com.sun.org.apache.xml.@internal.dtm.@ref;
using com.sun.xml.@internal.bind.v2.model.core;
using Elight.Entity;
using Elight.Logic;
using Elight.Utility;
using Elight.Utility.Code;
using Elight.Utility.Extensions;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using sun.net.www.content.image;

namespace _24Hour.Controllers.Common
{
    /// <summary>
    /// 工作台
    /// </summary>
    [Authorize]
    [ApiController]
    [Route("api/Staging")]
    public class StagingController : Controller
    {
        #region Identity
        private readonly SqlSugarClient _db;//数据库
        private readonly WriteSysLog _logs;//操作日志
        App_Sys_UserModel _userdata = new App_Sys_UserModel();//当前用户
        private readonly ILogger<LoginController> _logger;//日志
        Result result = new Result();
        public StagingController(ILogger<LoginController> logger, SqlSugarClient db, WriteSysLog logs, User user)
        {
            _logger = logger;
            _db = db;
            _logs = logs;
            _userdata = user.Userdata();
        }

        #endregion

        #region 工作台管理
        /// <summary>
        /// 工作台分页查询
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("QueryStaging")]
        public async Task<Result> QueryStaging(StagingInput Staging)
        {
            RefAsync<int> totalNumber = 0;//总数据
            var list = await _db.Queryable<StagingModel>()
                .WhereIF(!string.IsNullOrEmpty(Staging.title), q => q.title.Contains(Staging.title))
                .WhereIF(!string.IsNullOrEmpty(Staging.titletype), q => q.titletype.Contains(Staging.titletype))
                .Where(q => q.IsDelete == 0).ToPageListAsync(Staging.PageIndex, Staging.PageSize, totalNumber);
            Staging.RowsCount = totalNumber;
            var data = new QueryResult<StagingModel>(Staging, list);
            result.IsSucceed = true;
            result.result = data;
            return result;
        }
        /// <summary>
        /// 添加工作台
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("AddStaging")]
        public async Task<Result> AddStaging(StagingModel Stagingdata)
        {
            try
            {
                _db.BeginTran();
                Stagingdata.Id = Guid.NewGuid().ToString();
                Stagingdata.userId = _userdata.Id.ToString();
                Stagingdata.username = _userdata.name;
                var num = await _db.Insertable(Stagingdata).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "添加成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
            }
            _logs.WriteSysLogadd("工作台添加", "添加工作台", result, _db);
            return result;
        }

        /// <summary>
        /// 修改工作台
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("UpdateStaging")]
        public async Task<Result> UpdateStaging(StagingModel Stagingdata)
        {
            try
            {
                _db.BeginTran();
                var num = await _db.Updateable(Stagingdata).IgnoreColumns(ignoreAllNullColumns: true).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "修改成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
            }
            _logs.WriteSysLogadd("工作台", "修改工作台信息", result, _db);
            return result;
        }

        /// <summary>
        /// 删除工作台
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("DeleteStaging")]
        public async Task<Result> DeleteStaging(CurrencyDelete Currency)
        {
            try
            {
                _db.BeginTran();
                var Staginglist = await _db.Queryable<StagingModel>().In(q => q.Id, Currency.id).ToListAsync();
                Staginglist.ForEach(q =>
                {
                    q.IsDelete = 1;
                });
                var num = await _db.Updateable(Staginglist).ExecuteCommandAsync();
                _db.CommitTran();
                if (num > 0)
                {
                    result.IsSucceed = true;
                    result.result = "删除成功";
                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
            }
            _logs.WriteSysLogadd("工作台", "删除工作台信息", result, _db);
            return result;
        }

        /// <summary>
        /// 添加工作台使用次数
        /// </summary>
        /// <param name="info"></param>
        /// <returns></returns>
        [HttpGet]
        [Route("AddStagingnum")]
        public async Task<Result> AddStagingnum(string id)
        {
            try
            {
                var Stagingdata = await _db.Queryable<StagingModel>().Where(q => q.Id == id).FirstAsync();
                if (Stagingdata != null)
                {
                    var Stagingnumdata = await _db.Queryable<StagingNumModel>().Where(q => q.userId == _userdata.Id.ToString() && q.stagingId == id).FirstAsync();
                    if (Stagingnumdata != null)
                    {
                        Stagingnumdata.num++;
                        Stagingnumdata.creationtime = DateTime.Now;
                        _db.BeginTran();
                        var num = await _db.Updateable(Stagingnumdata).UpdateColumns(it => new { it.num }).ExecuteCommandAsync();
                        _db.CommitTran();
                        if (num > 0)
                        {
                            result.IsSucceed = true;
                            result.result = "";
                        }
                    }
                    else
                    {
                        _db.BeginTran();
                        var da = new StagingNumModel();
                        da.Id = Guid.NewGuid().ToString();
                        da.stagingId = id;
                        da.num = 1;
                        da.userId = _userdata.Id;
                        da.username = _userdata.name;
                        var num = _db.Insertable(da).ExecuteCommand();
                        _db.CommitTran();
                        if (num > 0)
                        {
                            result.IsSucceed = true;
                            result.result = "";
                        }
                    }
                }
                else
                {

                }
            }
            catch (System.Exception ex)
            {
                _db.RollbackTran();
                result.IsSucceed = false;
                result.Message = ex.Message;
            }
            _logs.WriteSysLogadd("工作台_使用数据", "更新或则添加使用次数", result, _db);
            return result;
        }
        /// <summary>
        /// 获取使用平凡的四个工作台信息
        /// </summary>
        /// <returns></returns>
        [HttpGet]
        [Route("GetStaginglist")]
        public async Task<Result> GetStaginglist()
        {
            var nulist = new List<StagingModel>();
            var Staginglist = await _db.Queryable<StagingModel>().Where(q => q.IsDelete == 0).ToListAsync();
            var Staginggrlist = Staginglist.GroupBy(q => q.titletype).Select(x => new { type = x.Key, data = x.ToList() }).ToList();
            var Numlist = await _db.Queryable<StagingNumModel>().Where(q => q.IsDelete == 0 && q.userId == _userdata.Id).ToListAsync();
            if (Numlist.Count > 0)
            {
                //获取常用工作台菜单
                var da = Numlist.OrderByDescending(q => q.creationtime).Take(10).Select(q => q.stagingId);
                var dalist = Staginglist.Where(q => da.Contains(q.Id)).ToList();

                //获取使用次数最多4个常用工作台菜单
                var Numda = Numlist.OrderByDescending(q => q.num).Take(4).Select(q => q.stagingId);
                nulist = Staginglist.Where(q => Numda.Contains(q.Id)).ToList();
                nulist.AddRange(Staginglist.Where(q => !Numda.Contains(q.Id)).Take(4 - nulist.Count()).ToList());
            }
            else
            {
                nulist = Staginglist.Take(4).ToList();
            }
            result.IsSucceed = true;
            result.result = new { typelist = Staginggrlist, numlist = nulist };
            return result;
        }
        #endregion

    }
}
