using Elight.Logic;
using Elight.Utility;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.Extensions.DependencyInjection.Extensions;
using Microsoft.IdentityModel.Tokens;
using Microsoft.OpenApi.Models;
using SqlSugar;
using System.Text;
using _24Hour.Controllers.Common;
using System.IdentityModel.Tokens.Jwt;
using Quartz.Impl;
using Quartz;
using Elight.Logic.Job;
using Quartz.Spi;
using Microsoft.IdentityModel.Logging;
using System.Collections.Specialized;
using System.Runtime.InteropServices;
using AutoMapper;
using System.Reflection;
using Elight.Entity;
using _24Hour;
using Microsoft.Extensions.DependencyInjection;
using _24Hour.Filter;
using Elight.Utility.Encrypt;

#region builder

Console.Title = "24HourServer";
var builder = WebApplication.CreateBuilder(args);

var Configuration = builder.Configuration;
builder.WebHost.UseUrls(Configuration.GetSection("UrlsConfiguration:Urls").Value.Split(","));
// Add services to the container.
#region


AutoMapper.IConfigurationProvider config = new MapperConfiguration(cfg =>
{
    cfg.AddProfile<AuthManageProfile>();
});
builder.Services.AddSingleton(config);
builder.Services.AddScoped<IMapper, Mapper>();
#endregion
builder.Services.AddControllers(ops =>
{
    ops.Filters.Add<RequestLoggingFilter>();
}).AddJsonOptions(options =>
{
    options.JsonSerializerOptions.PropertyNamingPolicy = null;
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.DateTimeNullableConverter());
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.NullableConverter<long>());
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.NullableConverter<int>());
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.NullableConverter<double>());
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.NullableConverter<decimal>());
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.NullableConverter<float>());
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.NullableConverter<Guid>());
    options.JsonSerializerOptions.Converters.Add(new Elight.Utility.NullableConverter<bool>());


});
// Learn more about configuring Swagger/OpenAPI at https://aka.ms/aspnetcore/swashbuckle
builder.Services.AddEndpointsApiExplorer();


builder.Host.ConfigureLogging(opt =>
{
});
#region Swaggerļ
builder.Services.AddSwaggerGen(c =>
{
    c.SwaggerDoc("v1", new OpenApiInfo { Title = "24Сʱһ API", Version = "v1" });
    //ڽӿࡢ [HiddenApi]ֹSwaggerĵ 
    c.DocumentFilter<HiddenApiFilter>();
    c.OrderActionsBy(o => o.RelativePath);
    var basePath = System.AppDomain.CurrentDomain.BaseDirectory;//ȡӦóĿ¼ԣܹĿ¼Ӱ죬ô˷ȡ·
    var xmlPath = Path.Combine(basePath, "24Hour.xml");
    var xmlPathentity = Path.Combine(basePath, "Elight.Entity.xml");
    var xmlPathlogic = Path.Combine(basePath, "Elight.Logic.xml");
    var xmlDto = Path.Combine(basePath, "Dto.xml");
    if (File.Exists(xmlPath))//ûиļʱ
        c.IncludeXmlComments(xmlPath, true);
    if (File.Exists(xmlPathentity))//ûиļʱ
        c.IncludeXmlComments(xmlPathentity, true);
    if (File.Exists(xmlPathlogic))//ûиļʱ
        c.IncludeXmlComments(xmlPathlogic, true);
    if (File.Exists(xmlDto))//ûиļʱ
        c.IncludeXmlComments(xmlDto, true);

    //Jwt֤
    c.AddSecurityRequirement(new OpenApiSecurityRequirement()
                        {
                            {
                                new OpenApiSecurityScheme
                                {
                                    Reference = new OpenApiReference
                                    {
                                        Id = "Bearer",
                                        Type = ReferenceType.SecurityScheme
                                    }
                                },
                                new List<string>()
                            }
                        });

    c.AddSecurityDefinition("Bearer", new OpenApiSecurityScheme
    {
        Description = "Value: Bearer {token}",
        Name = "Authorization",
        In = ParameterLocation.Header,
        Type = SecuritySchemeType.ApiKey
    });
});
#endregion
// ÿ
builder.Services.AddCors(policy =>
{
    policy.AddPolicy("CorsPolicy", opt => opt
    .AllowAnyOrigin()
    .AllowAnyHeader()
    .AllowAnyMethod()
    .WithExposedHeaders("X-Pagination"));
});


//builder.Services.AddCors(options => options.AddPolicy("CorsPolicy",
//c =>
//{
//    c.WithOrigins(Configuration.GetSection("UrlsConfiguration:CorUrls").Value.Split(',', StringSplitOptions.RemoveEmptyEntries))
//           .AllowAnyHeader() // καͷ(дAllowAnyMethodȥ)
//           .AllowAnyMethod() // κη
//           .SetIsOriginAllowed(o => true) // =AllowAnyOrigin()
//           .AllowCredentials();
//}));
//jwt toekn
JwtSecurityTokenHandler.DefaultInboundClaimTypeMap.Clear();
JwtSecurityTokenHandler.DefaultOutboundClaimTypeMap.Clear();
// ֤
builder.Services.AddAuthentication(options =>
{
    options.DefaultScheme = JwtBearerDefaults.AuthenticationScheme;
    options.DefaultChallengeScheme = nameof(ResponseAuthenticationHandler); //401
    options.DefaultForbidScheme = nameof(ResponseAuthenticationHandler);    //403
})
.AddJwtBearer(options =>
{
    // JWT֤ѡ
    options.TokenValidationParameters = new TokenValidationParameters
    {
        ValidateIssuer = true,
        ValidateAudience = true,
        ValidateLifetime = true,
        ValidateIssuerSigningKey = true,
        ValidIssuer = Configuration.GetSection("JwtConfiguration:Issuer").Value,
        ValidAudience = Configuration.GetSection("JwtConfiguration:Audience").Value,
        IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(Configuration.GetSection("JwtConfiguration:Jwtkey").Value)),
        ClockSkew = TimeSpan.Zero
    };

    //صжWebSocket·(https://www.cnblogs.com/fger/p/11811190.html)
    options.Events = new JwtBearerEvents
    {
        OnMessageReceived = (context) =>
        {
            if (!context.HttpContext.Request.Path.HasValue) return Task.CompletedTask;
            //صжWebSocket·
            var accessToken = context.HttpContext.Request.Query["access_token"];
            var path = context.HttpContext.Request.Path;
            if (string.IsNullOrWhiteSpace(accessToken) || !path.StartsWithSegments("/ws")) return Task.CompletedTask;

            context.Token = accessToken;
            return Task.CompletedTask;
        },

        //˴ΪȨ֤ʧܺ󴥷¼
        OnChallenge = context =>
        {
            //˴Ϊֹ.Net CoreĬϵķͺݽҪŶ
            context.HandleResponse();
            //ԶԼҪصݽҪصJsonͨNewtonsoft.Jsonת
            var payload = new { StatusCode = 0, Message = "֤ʧܣ" };
            //Զ巵ص
            context.Response.ContentType = "application/json";
            //Զ巵״̬룬ĬΪ401 ĳ 200
            context.Response.StatusCode = StatusCodes.Status200OK;
            //context.Response.StatusCode = StatusCodes.Status401Unauthorized;
            //Jsonݽ
            context.Response.WriteAsync(Convert.ToString(payload));
            return Task.FromResult(0);
        }
    };

}).AddScheme<AuthenticationSchemeOptions, ResponseAuthenticationHandler>(nameof(ResponseAuthenticationHandler), o => { });

builder.Services.AddHttpClient<WechatMessagerClient>(opt =>
{
    opt.BaseAddress = new Uri("https://api.weixin.qq.com/sns");
});
builder.Services.AddHttpClient<TwentySystemProxyClient>(opt =>
{
    opt.BaseAddress = new Uri(Configuration.GetSection("CaseTwenty:SysAddress").Value);
});

//builder.Services.AddAuthorization();
builder.Services.AddHttpContextAccessor();
builder.Services.AddScoped<WebSocketController>();
builder.Services.TryAddSingleton<User, User>();
builder.Services.TryAddSingleton<WriteSysLog, WriteSysLog>(); //WriteSysLog
builder.Services.AddScoped<SqlSugarClient>(sp =>
{
    var config = Configuration.GetSection("ConnectionStrings:MySQLConnString").Value;
    var connectionString = AES.Decrypt(config);
    var db = new SqlSugarClient(new ConnectionConfig
    {
        ConnectionString = connectionString,
        DbType = SqlSugar.DbType.MySql,
        IsAutoCloseConnection = true,
        InitKeyType = InitKeyType.Attribute
    });
    return db;
});
//httpclient ־¼
builder.Services.AddHttpLogging(logging =>
{
    logging.LoggingFields = Microsoft.AspNetCore.HttpLogging.HttpLoggingFields.All;
});

#region ʱ
// Quartz.NET
builder.Services.TryAddSingleton<IJobFactory, JobFactory>();
builder.Services.TryAddSingleton<ISchedulerFactory, StdSchedulerFactory>();
builder.Services.TryAddSingleton<WsJob>();
builder.Services.TryAddSingleton<YgJob>();
builder.Services.AddHostedService<QuartzHostedService>(); // עQuartz.NETйܷ



builder.Logging.AddLog4Net("log4net.config");


//Task.Factory.StartNew(() =>
//{
//    try
//    {
//        while (true)
//        {
//            Thread.Sleep(60 * 1000);
//            ClearMemory();
//        }
//    }
//    catch (Exception ex)
//    {
//    }
//});
//Task.Factory.StartNew(() =>
//{
//    RunJob().Wait();
//});
#endregion
#endregion

#region APP
var app = builder.Build();

// Configure the HTTP request pipeline.
if (app.Environment.IsDevelopment())
{
    app.UseSwagger();// Swaggerм
    app.UseSwaggerUI(c =>
    {
        c.SwaggerEndpoint("/swagger/v1/swagger.json", "24Сʱһ API V1");
        c.RoutePrefix = string.Empty;

    });
}
#region websockets
app.UseWebSockets(new WebSocketOptions
{
    KeepAliveInterval = TimeSpan.FromMinutes(2)
});
//app.UseMiddleware<WebSocketMiddleware>();
#endregion
app.UseStaticFiles();
//app.UseHttpsRedirection();
//·

//ʹhttpclient־¼ UseStaticFiles
app.UseHttpLogging();

app.UseRouting();
app.UseAuthentication(); // ֤м
app.UseAuthorization(); // Ȩм
//app.MapControllers();
app.UseEndpoints(endpoints =>
{
    endpoints.MapControllers();
});

app.UseCors("CorsPolicy");
app.Run();
#endregion


#region ʱ
static void ClearMemory()
{
    try
    {
        GC.Collect();
        GC.WaitForPendingFinalizers();
    }
    catch (Exception ex)
    {
    }
}
static async Task RunJob()
{
    try
    {
        NameValueCollection props = new NameValueCollection
                {
                    { "quartz.serializer.type", "binary" }
                };
        StdSchedulerFactory factory = new StdSchedulerFactory(props);
        IScheduler scheduler = await factory.GetScheduler();
        //   
        await scheduler.Start();
        #region ͨϢÿ33ִ3
        IJobDetail job3 = JobBuilder.Create<WsJob>()
                              .WithIdentity("job3", SchedulerConstants.DefaultGroup)
                              .Build();
        var trigger3 = TriggerBuilder.Create()
                        .WithIdentity("trigger3") // һ  
                        .StartAt(DateTime.Now) // ʼʱ  
                        .ForJob("job3", SchedulerConstants.DefaultGroup) //ָһ  
                        .WithCronSchedule("*/20 * * * * ?")
                        .UsingJobData("k", 321)
                        .Build();
        await scheduler.ScheduleJob(job3, trigger3);
        #endregion
    }
    catch (SchedulerException se)
    {
        await Console.Error.WriteLineAsync(se.ToString());
    }
}
#endregion
