﻿using System;
using System.Runtime.Serialization;

namespace Elight.Utility.Code
{
    /// <summary>
    /// 分页类
    /// </summary>
    [Serializable]
    [DataContract]
    public class Paging
    {
        /// <summary>
        /// 页码
        /// </summary>
        [DataMember]
        private int pageIndex = 1;

        /// <summary>
        /// 页码
        /// </summary>
        [DataMember]
        public int PageIndex
        {
            get
            {
                return this.pageIndex;
            }

            set
            {
                this.pageIndex = value;
            }
        }

        /// <summary>
        /// 页大小
        /// </summary>
        [DataMember]
        private int pageSize = 10;

        /// <summary>
        /// 页大小(默认10页)
        /// </summary>
        [DataMember]
        public int PageSize
        {
            get
            {
                return this.pageSize;
            }

            set
            {
                this.pageSize = value;
            }
        }

        /// <summary>
        /// 总条数
        /// </summary>
        [DataMember]
        public int RowsCount { get; set; }

        /// <summary>
        /// 总页数
        /// </summary>
        [DataMember]
        private int pageCount;

        /// <summary>
        /// 总页数
        /// </summary>
        [DataMember]
        public int PageCount
        {
            get
            {
                if (PageSize == 0)
                {
                    return 0;
                }
                this.pageCount = (this.RowsCount % this.PageSize) == 0
                                     ? this.RowsCount / this.PageSize
                                     : (this.RowsCount / this.PageSize) + 1;
                return this.pageCount;
            }

            set
            {
                this.pageCount = value;
            }
        }

        /// <summary>
        /// 是否获取总条数
        /// </summary>
        [DataMember]
        private bool getRowsCount = true;

        /// <summary>
        /// 排序名
        /// </summary>
        [DataMember]
        public string? SortName { get; set; }

        /// <summary>
        /// 排序方式
        /// </summary>
        [DataMember]
        public string? SortType { get; set; }

        /// <summary>
        /// 实名数量
        /// </summary>
        [DataMember]
        public int RealNameCount { get; set; }

        /// <summary>
        /// 匿名数量
        /// </summary>
        [DataMember]
        public int AnonymousCount { get; set; }


        /// <summary>
        /// 开始时间
        /// </summary>
        [DataMember]
        public DateTime? StartTime { get; set; }


        /// <summary>
        /// 结束时间
        /// </summary>
        [DataMember]
        public DateTime? EndTime { get; set; }

        /// <summary>
        /// 是否获取总条数
        /// </summary>
        [DataMember]
        public bool GetRowsCount
        {
            get
            {
                return this.getRowsCount;
            }

            set
            {
                this.getRowsCount = value;
            }
        }

        /// <summary>
        /// 开始索引
        /// </summary>
        public int StratRows => PageSize * ((PageIndex <= 0 ? 1 : PageIndex) - 1) + 1;

        /// <summary>
        /// 开始索引
        /// </summary>
        public int EndRows => (PageIndex <= 0 ? 1 : PageIndex) * PageSize;
    }
}